/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <string>

#include "lmdb.h"
#include "base.h"
#include "storage.h"
#include "transaction.h"
#include "cursorcommon.h"

namespace LMDBAL {

template <class K, class V>
class Cursor : public CursorCommon {
    friend class Storage<K, V>;
public:
    Cursor();
    Cursor(Storage<K, V>* parent);
    Cursor(const Cursor& other) = delete;
    Cursor(Cursor&& other);
    ~Cursor();

    Cursor& operator = (const Cursor& other) = delete;
    Cursor& operator = (Cursor&& other);

    void drop();

    std::pair<K, V> first();
    std::pair<K, V> last();
    std::pair<K, V> next();
    std::pair<K, V> prev();
    std::pair<K, V> current() const;
    bool set(const K& target);

    void first(K& key, V& value);
    void last(K& key, V& value);
    void next(K& key, V& value);
    void prev(K& key, V& value);
    void current(K& key, V& value) const;

private:
    void operateCursorRead(K& key, V& value, MDB_cursor_op operation, const std::string& methodName, const std::string& operationName) const;
};

};


#include "cursor.hpp"
