/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once


#include <cstring>

#include <QByteArray>
#include <QBuffer>
#include <QDataStream>

#include <lmdb.h>

namespace LMDBAL {

template<class T>
class Serializer {
public:
    Serializer();
    Serializer(const T& value);
    ~Serializer();

    T deserialize(const MDB_val& value);
    void deserialize(const MDB_val& value, T& result);
    MDB_val setData(const T& value);
    MDB_val getData();
    void clear();

private:
    void _setData(const T& value);


private:
    QByteArray bytes;
    QBuffer buffer;
    QDataStream stream;
};

}

#include "serializer.hpp"
#include "serializer_uint64.hpp"
#include "serializer_uint32.hpp"
#include "serializer_uint16.hpp"
#include "serializer_uint8.hpp"
#include "serializer_int64.hpp"
#include "serializer_int32.hpp"
#include "serializer_int16.hpp"
#include "serializer_int8.hpp"
#include "serializer_float.hpp"
#include "serializer_double.hpp"
#include "serializer_stdstring.hpp"
#include "serializer_qstring.hpp"
#include "serializer_qbytearray.hpp"
