/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <stdint.h>

namespace LMDBAL {

template<>
class Serializer<int8_t> {
public:
    Serializer():value(0) {};
    Serializer(const int8_t& p_value):value(p_value) {};
    ~Serializer() {};

    int8_t deserialize(const MDB_val& data) {
        deserialize(data, value);
        return value;
    };
    void deserialize(const MDB_val& data, int8_t& result) {
        std::memcpy(&result, data.mv_data, 1);
    }
    MDB_val setData(const int8_t& data) {
        value = data;
        return getData();
    };
    MDB_val getData() {
        MDB_val result;
        result.mv_data = &value;
        result.mv_size = 1;
        return result;
    };
    void clear() {};    //not possible;

private:
    int8_t value;
};

}
