/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QString>
#include <QByteArray>

namespace LMDBAL {

template<>
class Serializer<QString> {
public:
    Serializer():value() {};
    Serializer(const QString& p_value):value(p_value.toUtf8()) {};
    ~Serializer() {};

    QString deserialize(const MDB_val& data) {
        value = QByteArray((char*)data.mv_data, data.mv_size);
        return QString::fromUtf8(value);
    };
    void deserialize(const MDB_val& data, QString& result) {
        value = QByteArray((char*)data.mv_data, data.mv_size);
        result = QString::fromUtf8(value);
    }
    MDB_val setData(const QString& data) {
        value = data.toUtf8();
        return getData();
    };
    MDB_val getData() {
        MDB_val result;
        result.mv_data = value.data();
        result.mv_size = value.size();
        return result;
    };
    void clear() {};    //not possible;

private:
    QByteArray value;
};

}
