/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "base.h"

namespace LMDBAL {
class StorageCommon;
class CursorCommon;

class Transaction {
    friend class Base;
    friend class StorageCommon;
public:
    explicit Transaction();
    explicit Transaction(Transaction&& other);
    Transaction(const Transaction& other) = delete;
    Transaction& operator = (const Transaction& other) = delete;
    Transaction& operator = (Transaction&& other);
    virtual ~Transaction();

    void terminate();
    bool isActive() const;

protected:
    Transaction(TransactionID txn, const Base* parent);
    void reset();
    void closeCursors();

protected:
    TransactionID txn;                          /**<\brief Transaction inner handler*/
    bool active;                                /**<\brief Transaction state*/
    const Base* parent;                         /**<\brief Pointer to the database this transaction belongs to*/
    std::map<uint32_t, CursorCommon*> cursors;  /**<\brief a collection of cursors curently opened under this transaction*/
};

class WriteTransaction : public Transaction {
    friend class Base;
public:
    explicit WriteTransaction();
    explicit WriteTransaction(WriteTransaction&& other);
    WriteTransaction(const WriteTransaction& other) = delete;
    WriteTransaction& operator = (const WriteTransaction& other) = delete;
    WriteTransaction& operator = (WriteTransaction&& other);

    void commit();
    void abort();

protected:
    WriteTransaction(TransactionID txn, Base* parent);
};

}
